
%CreateTestSampleData.m 
%Author: Sandra Escandor, 2010
%Purpose:
%To create a set of sample observation data that we can enter into
%the test application for the Sony Ericsson cell phone simulator
clc;
clear;
SampleSize = 100;
%This is test data to check if we are indeed getting the right values
%for Covariance and the test data points are being classified properly
%x = [4.0, 4.2, 3.9, 4.3, 4.1]';
%y = [2.0, 2.1, 2.0, 2.1, 2.2]';
%z = [0.6, 0.59,0.58,0.62,0.63]';

%Let's Generate some Gaussian random variables for our sample observation data 
%Notes:
%x=>Heart Rate
%y=>Temperature
%z=>Systolic blood pressure (came across research that said that systolic has the most meaning)
meanX = 80;
meanY = 37;
meanZ = 120;
stdDevX = 10;
stdDevY = 0.5;
stdDevZ = 0.5;
x = meanX + stdDevX.*randn(SampleSize,1); %mean at 80 with standard dev. of 10
y = meanY + stdDevY.*randn(SampleSize,1); %mean of 37 with standard dev. of 0.5
z = meanZ + stdDevZ.*randn(SampleSize,1); %mean of 120 with standard dev. of 0.5

sampleData = [x y z];



X = sampleData;
[rowLengthX colLengthX] = size(X);
%Create the mean vector of the sample observations
meanVec = [ sum(X(:,1))/rowLengthX sum(X(:,2))/rowLengthX sum(X(:,3))/rowLengthX]'

chi_maxdist_sq = 7.815;
chi_warningdist_sq = 6.251;

%For matrices, where each row is an observation, and each column is a variable, cov(X) is the covariance matrix.
%See: http://www.mathworks.com/access/helpdesk/help/techdoc/ref/cov.html
covX = cov(X)

%Show the inverse of this covariance matrix
Y = inv(covX)

% open the file with write permission
fid = fopen('exp.txt', 'w');
[rows cols] = size(sampleData);

fprintf(fid, 'Data = [');
for i = 1:rows
    if(i ~= rows)
        fprintf(fid, '%3.0f, %3.0f, %3.0f;', sampleData(i,:));
    else
        fprintf(fid, '%3.0f, %3.0f, %3.0f]', sampleData(i,:));
    end
end

fprintf(fid,'\n');

%generate java code for the sample data
fprintf(fid, 'double[][] digitized = {');
for i = 1:rows
    
    if(i ~= rows)
        fprintf(fid, '{%3.2f, %3.2f, %3.2f},', sampleData(i,:));
    else
        fprintf(fid, '{%3.2f, %3.2f, %3.2f}', sampleData(i,:));
    end
end
fprintf(fid, '};\n');

%generate the java code for the test data points
        
    %For an outlier:
    %1. Find the eigenvectors and eigenvalues of the covariance matrix.
    %   This gives us the vectors that support the ellipsoids for each quantile we're looking at. 
    %2. Look at the mean of the sample data: this is the center of the
    %   ellipsoids.
    %3. Take an eigenvector_A and its associated eigenvalue_A.
    %4. Multiply eignevector_A with the square root of
    %   (eignevalue_A*(chi_warningdist_sq+0.11) ). Note that the 0.11 is so
    %   we can be in the outlier region comfortably.
    %5. Add the mean vector the vector resulting from step 4 => we have our
    %   outlier data point!
    
    [V, D] = eig(covX);
    AddThisVectorToMeanVector = sqrt((chi_maxdist_sq+0.11)*D(1,1)) .*V(:,1);
        
    outlierDataPt = (meanVec + AddThisVectorToMeanVector);
    fprintf(fid, 'double[] point1 = { %3.2f, %3.2f, %3.2f }; //Outlier Data Test Point \n', outlierDataPt(1), outlierDataPt(2), outlierDataPt(3));

    %For a regular data point, just use one of the data points from the
    %sample data
    AddThisVectorToMeanVector = sqrt((chi_warningdist_sq-0.5)*D(2,2)) .*V(:,2);
    RegularDataPt = sampleData(1,:)'
    fprintf(fid, 'double[] point2 = { %3.2f, %3.2f, %3.2f }; //Regular Data Test Point \n', sampleData(1,:));

    %For a warning data point: 
    %1. Find the eigenvectors and eigenvalues of the covariance matrix.
    %   This gives us the vectors that support the ellipsoids for each quantile we're looking at. 
    %2. Look at the mean of the sample data: this is the center of the
    %   ellipsoids.
    %3. Take an eigenvector_A and its associated eigenvalue_A.
    %4. Multiply eignevector_A with the square root of
    %   (eignevalue_A*(chi_warningdist_sq+0.11) ). Note that the 0.11 is so
    %   we can be in the warning region comfortably.
    %5. Add the mean vector the vector resulting from step 4 => we have our warning
    %   data point!
        
    AddThisVectorToMeanVector = sqrt((chi_warningdist_sq+0.11)*D(3,3)) .*V(:,3);
    
    warningDataPt = (meanVec + AddThisVectorToMeanVector);
    
    fprintf(fid, 'double[] point3 = { %3.2f, %3.2f, %3.2f }; //Warning Data Test Point\n', warningDataPt(1), warningDataPt(2), warningDataPt(3));
    
    
fclose(fid);

% view the contents of the file
type exp.txt


Xtrans = X';
%Draw a scatter plot
col1 = [Xtrans(1,:)];
col2 = [Xtrans(2,:)];
col3 = [Xtrans(3,:)];
scatter3(col1,col2,col3), title('Data cluster for 100 generated sample observations.');

%Do one point test that we know should be considered an outlier
intermediate = outlierDataPt - meanVec;
D_sq_Outlier = (intermediate')*(Y)*(intermediate)
chi_warningdist_sq
chi_maxdist_sq


if D_sq_Outlier < chi_warningdist_sq
    text = 'Classification: Regular Point. Should be an Outlier!'
else
    
    if D_sq_Outlier >= chi_warningdist_sq && D_sq_Outlier < chi_maxdist_sq
        text = 'Classification: Warning Point. Should be an Outlier'
    end
    
    if D_sq_Outlier >= chi_maxdist_sq
        text = 'Classification: Outlier. Matches our expectation! Remember: If this is still WITHIN the monitoring physician''s chosen observation range, it will just be considered a warning in the vital stats monitor!'
    end
end

%Do one point test that we know should be a regular point
%Note that this is one of the points in the observation matrix X
intermediate = RegularDataPt - meanVec;
D_sq_Regular = (intermediate')*(Y)*(intermediate)
chi_warningdist_sq
chi_maxdist_sq


if D_sq_Regular < chi_warningdist_sq
    text = 'Classification: Regular Point. Matches our expectation!'
else
    
    if D_sq_Regular >= chi_warningdist_sq && D_sq_Regular < chi_maxdist_sq
        text = 'Classification: Warning Point. Should be Regular Point!'
    end
    
    if D_sq_Regular >= chi_maxdist_sq
        text = 'Classification: Outlier. Should be Regular Point!'
    end
end

%Do one point test that we know should be a warning point

intermediate = warningDataPt - meanVec;
D_sq_Warning = (intermediate')*(Y)*(intermediate)
chi_warningdist_sq
chi_maxdist_sq

if D_sq_Warning < chi_warningdist_sq
    text = 'Classification: Regular Point. Should be Warning Point!'
else
    
    if D_sq_Warning >= chi_warningdist_sq && D_sq_Warning < chi_maxdist_sq
        text = 'Classification: Warning Point. Matches our expectation!'
    end
    
    if D_sq_Warning >= chi_maxdist_sq
        text = 'Classification: Outlier. Should be Warning Point!'
    end
end



