

/**
 *
 * @author Sandra Escandor
 * The heart rate generated by ProcessingDigitized is sent to trivar_normal
 * This is a strip down version of the QRS Detection algorithm by Adnane, Jiang, and Choi
 * (See: "Development of QRS detection algorithm designed for wearable cardio respiratory system.")
 */
public class ProcessingDigitized {
    
    //Sample period of 0.004 is the sample period of record cdb/08730_01
    //From: http://physionet.org/cgi-bin/ATM
    private double samplePeriod = 0.004; //TEST LINE. Note: We're sampling at 200Hz so we'll need to change this when we get data from our own ECG. This sampling rate was for the test data that we used to test this program.
    private double posThresh; //We will use averaging here, plus a percentage offset (estimated).
    private double negThresh; //We will use averaging here, plus a percentage offset (estimated).
    private double[] digitized = null;
    private int cntPosThresh = 2; //the number of positive slope thresholds that need to be counted in order to find R peak
    private int cntNegThresh = 2;
    private int AllowableExamineRange = 15; //the range of number of samples that the negative slope needs to be found
    private int HeartRate = 0;
    
    private int FirstRPeakStartLoc = 0; 
    private int FirstRPeakEndLoc = 0; 
    private int SecondRPeakStartLoc = 0; 
    private int SecondRPeakEndLoc = 0; 
    
    public ProcessingDigitized(double[] digitized)
    {
        this.digitized = digitized;
        int cntr = 0;
        
        //Set the slope thresholds:
        posThresh = GetSlopeAverage(true);
        negThresh = GetSlopeAverage(false);
        //Add the offsets:
        posThresh += posThresh*2.5;
        negThresh += negThresh*2.5;
        
        FirstRPeakStartLoc = RPeakStartLoc(1,cntr);
        cntr = FirstRPeakStartLoc/3;
        FirstRPeakEndLoc = RPeakEndLoc(FirstRPeakStartLoc,cntr,FirstRPeakStartLoc);
        
        int PeakFirst = GetIndexOfMax(FirstRPeakStartLoc, FirstRPeakEndLoc);
        
        SecondRPeakStartLoc = RPeakStartLoc(FirstRPeakEndLoc+1,FirstRPeakEndLoc);
        cntr = SecondRPeakStartLoc/3;
        SecondRPeakEndLoc = RPeakEndLoc(SecondRPeakStartLoc,cntr,SecondRPeakStartLoc);
        
        int PeakSecond = GetIndexOfMax(SecondRPeakStartLoc,SecondRPeakEndLoc);
        
        double RRInt = (PeakSecond-PeakFirst)*samplePeriod;
        
        double HeartRateExact = (60/RRInt);
        int HeartRateCeiling = (int)Math.ceil(HeartRateExact);
        
        //Do the rounding.
        if(1-(double)(HeartRateCeiling-HeartRateExact) < 0.5)
        {
            HeartRate = (int)Math.floor(HeartRateExact);
        }
        else
        {
            HeartRate = (int)Math.ceil(HeartRateExact);
        }
        
    }
    
    public int GetFirstRPeakStartLoc()
    {
        return this.FirstRPeakStartLoc;
    }
    
    public int GetFirstRPeakEndLoc()
    {
        return this.FirstRPeakEndLoc;
    }
    
    public int GetSecondRPeakStartLoc()
    {
        return this.SecondRPeakStartLoc;
    }
    
    public int GetSecondRPeakEndLoc()
    {
        return this.SecondRPeakEndLoc;
    }
    
    public double GetPosSlopeThresh()
    {
        return this.posThresh;
    }
    
    public double GetNegSlopeThresh()
    {
        return this.negThresh;
    }
    //Use the average slope of the given data to determine the positive
    //and negative slope thresholds.
    private double GetSlopeAverage(boolean positive)
    {
        double slopeAvg = 0;
        int numType = 0; //For averaging
        double slope = 0;
        int ExamineAtIndex = 1;
        
        if(positive) //Looking for positive slope average
        {
            int m = 1;
            while(ExamineAtIndex+2 < digitized.length)
            {
               slope = (digitized[ExamineAtIndex+1]-digitized[ExamineAtIndex-1])/this.samplePeriod;
               
               if(slope > 0)
               {
                   numType++;
                   slopeAvg += slope;
               }
               
               
               ExamineAtIndex = 2*m;
               m++;
            }
               
        }
        else //Looking for negative slope average
        {
            int m = 1;
            while(ExamineAtIndex+2 < digitized.length)
            {
               slope = (digitized[ExamineAtIndex+1]-digitized[ExamineAtIndex-1])/this.samplePeriod;
               
               if(slope < 0)
               {
                   numType++;
                   slopeAvg += slope;
               }
               
               ExamineAtIndex = 2*m;
               m++;
            }
            
        }
        
        slopeAvg /= numType;
        
        return slopeAvg;
    }
    
    
    public int GetHeartRate()
    {
        return this.HeartRate;
    }
    
    //Let's look for the peak with the largest positive slope characteristic.
    //This will be the R peak.
    private int RPeakStartLoc(int ExamineAtIndex, int cntr)
    {
        int posSlopeCnt = 0;
        
        double slope = 0;
                
        //We want to look at indices 1,3,6,...etc. instead of the 2,4,6,... 
        //pattern in the matlab trial code, since the start value of an array's
        //index is 0 here.
                
        while( (posSlopeCnt < cntPosThresh) && (ExamineAtIndex+2 < digitized.length) ) 
        {   
            
            slope = (digitized[ExamineAtIndex+1]-digitized[ExamineAtIndex-1])/this.samplePeriod;
            
            if(slope >= posThresh)
            {       
                if(posSlopeCnt > 0)
                    posSlopeCnt = posSlopeCnt + 1;
                else
                { 
                    if(posSlopeCnt == 0)
                        posSlopeCnt = 1;
                    else                
                        posSlopeCnt = 0; //reset the slope count so we can find a consecutive set of slopes that are at least above the threshold
                }
                
            }
            else
            {
                posSlopeCnt = 0;
            }
            
            //Let's change where we're looking in the array
            cntr+=1;
            ExamineAtIndex = 2*cntr;
        }
        
        return ExamineAtIndex;
    }
    
    //After finding the RPeak start location, we would like to know where
    //the RPeak starts to go down. This will be useful in order to get
    //the R peak location.
    private int RPeakEndLoc(int ExamineAtIndex, int cntr, int RPeakStartLoc)
    {
         int RPeakEndLoc = 0;
         int negSlopeCnt = 0;
         double slope = 0;
         
         while(negSlopeCnt < cntNegThresh && ExamineAtIndex+2 < digitized.length && ExamineAtIndex-RPeakStartLoc < AllowableExamineRange)
         {
         
            slope = (digitized[ExamineAtIndex+1]-digitized[ExamineAtIndex-1])/this.samplePeriod;
            
            if (slope <= negThresh)
            {
                if(negSlopeCnt > 0)
                {
                    negSlopeCnt = negSlopeCnt + 1;
                }
                else
                {

                    if (negSlopeCnt == 0)
                        negSlopeCnt = 1;
                    else                
                        negSlopeCnt = 0; //reset the slope count so we can find a consecutive set of slopes that are at least above the threshold
                    
                }
            }
            else
            {
                negSlopeCnt = 0;
            }
            
            //Let's change where we're looking in the array and move over two spaces from
            //the current element in the array that we're examining
            cntr+=1;
            ExamineAtIndex = 2*cntr;
         }
                  
         return ExamineAtIndex;
    }
    
    //Since we now know where the peak starts and peak ends,
    //we can find the maximum point between these two in order to get the
    //R location.
    private int GetIndexOfMax(int Start, int End)
    {
        int IndexOfMax = 0;
        double maxVal = 0;
        
        for(int i = Start; i<End; i++)
        {
            if(digitized[i] > maxVal)
            {
                maxVal = digitized[i];
                IndexOfMax = i;
            }
        }
        
        return IndexOfMax; 
    }
}
