
/**
 *public class trivar_normal
 *This is the class that handles the trivariate normal analysis.
 *It has two modes: 
 *1. training
 *2. functional
 *In training mode - Takes in the first 24 hours worth of data from the transmitter.
 *Once it has obtained this data, it will calculate the mean matrix and the covariance matrix.
 *After the calculation of the two matrices, it will switch to functional mode.
 *Inputs:    
 *   In training mode:
 *       training_matrix - Is a matrix created from the n-rows, three column CSV data that is transmitted from the sensors
 *   In functional mode:
 *       data_matrix - Is a matrix created from the 1-row, three column CSV data that is transmitted from the sensors
 *Outputs:
 *   In functional mode:
 *       Outlier -  True if a 1-row, three column matrix input from transmitter is considered an outlier (using Mahalanobis distance)      
 *
 * IMPORTANT NOTE:
 * See: http://www.itl.nist.gov/div898/handbook/pmc/section5/pmc541.htm 
 * for the idea behind calculating the covariance matrix and the mean matrix. Thanks.
 * 
 * See: http://individual.utoronto.ca/rav/FR/cov.htm
 * for clarification on covariance matrix (if the itl.nist.gov website confuses you)
 * 
 * @author Sandra Escandor, 2010.
 */
public class trivar_normal {
	
        /**
         * Variables:
         * 
         * FunctionalMode - Determines trivar_normal's operating mode (see the rest of the comments below for what I mean).
         * training_data - a matrix that has three columns and many many rows (number of rows that correspond to the number of data for 24 hours).
         *                 training_data is used to calculate the mean matrix and the covariance matrix.
         * mean_matrix - (this is a misnomer since it's a mean vector) a 3x1 matrix that has columns which correspond to the mean value of each column of training_data.
         * covar_matrix -  a 3x3 matrix that has variance of x, y, z along its diagonal, etc. see websites link at the top part of this documentation for more clarification.
         * 
         */
            
        private Boolean FunctionalMode = Boolean.FALSE; 
	private Matrix training_data;
	private Vektor point_data;
	private Matrix mean_matrix;
	private Matrix covar_matrix;
        private Matrix covarMatInverse; //We want to only calculate this once
	//Calculate Statistical Distance:
        //Source: "Statistical Methods In The Atmospheric Sciences" Table B.3
        //chi_sq_dist = the chi square distribution for 95% quantile with 3 degrees of freedom (since three dimensions)
        final double chi_sq_dist_outlier_sq = 7.8150;
        final double chi_sq_dist_warning_sq = 6.2510;
        
        //These are automatically set to the following values if the 
        //monitoring physician doesn't specifically set them:
        private double minHeartRate = 59.5;
        private double maxHeartRate = 96.0;
        private double minTemp = 34.7;  //Celsius!
        private double maxTemp = 40.0;
        private double minSystolic = 100;
        private double maxSystolic = 160;
        
        //This constructor needs the past observation data and its dimensions
        public trivar_normal(double[][] training_data, int numRows, int numCols) throws Exception
        {
            //Initialize all data owned by this instance of trivar_normal class
            this.mean_matrix = new Matrix(1,3);
            this.point_data = new Vektor(3);
            this.covar_matrix = new Matrix(3,3);
            
            this.training_data = new Matrix(training_data, numRows, numCols);
            
            try{
                                
                //Try calculating the mean vector (the mean of the data matrix)
                calcMeanMatrix();  
            }
            catch(Exception x)
            {
                //We need to handle the exception when mahalanobisDist() 
                //can't convert the matrix data type to double data type
                throw new Exception("Could not calculate the mean of the matrix. " + x.toString());
            }

            calcCovarMatrix();

            SetMode(Boolean.TRUE);        
        }
        
         //Determines if the point that was given to it has something wrong
         //Need to signal if there is something wrong.
         public int FunctionalResult() throws Exception
         {
             int result = 1;
             
             double distance_sq = mahalanobisDist();
             if(distance_sq >= this.chi_sq_dist_warning_sq && distance_sq < this.chi_sq_dist_outlier_sq)
             {
                 //This is data point is in the warning region
                 result = 2;
             }
             
             if(distance_sq >= this.chi_sq_dist_outlier_sq)
             {
                 
                 if(WithinSetRange() > 0) //This data point is in the danger region
                    result = 3;
                 else 
                    result = 2; //Still within the chosen observation range by monitoring physician. Just warn them.
             }
             
             //If the monitoring physician's set observation ranges have been 
             //surpassed, then we want to signal a warning.
             if(WithinSetRange() > 0)
             {
                 result = 2; 
             }
             
             return result;
                          
         }
         
         //We would like to check if the vital sign values are within
         //a prescribed range (prescribed by the monitoring physician)
         //Determine which value is out of range using binary, and the position
         //of the vital signs in the double[]:
         //000 => Nothing is out of range => return 0
         //001 => Systolic is out of range => return 1
         //010 => Temperature is out of range => return 2
         //011 => Both Systolic and Temperature out of range => return 3
         //100 => Heart rate is out of range => return 4
         //101 => Heart rate and Systolic are out of range => return 5
         //110 => Heart rate and Temperature are out of range => return 6
         //111 => All out of range, return 7
         private int WithinSetRange() throws Exception
         {
             
             double dblPointData[] = null;
             double DataHeartRate = 0;
             double DataTemp = 0;
             double DataSystol = 0;
             
             try{
                 
                 dblPointData = getPointData();
             }
             catch(Exception x)
             {
                 throw new Exception("Could not retrieve point_data as a double array while trying to determine range values in WithinSetRange(). " + x.toString());
             }
             
             DataHeartRate = dblPointData[0];
             DataTemp = dblPointData[1];
             DataSystol = dblPointData[2];
                         
             int WithinSetRangeDigit = 0;
             
             //Check Heart Rate
             if(DataHeartRate < this.minHeartRate || DataHeartRate > this.maxHeartRate )
             {
                 WithinSetRangeDigit += 4;
             }
             
             //Check Temperature
             if(DataTemp < this.minTemp || DataTemp > this.maxTemp)
             {
                 WithinSetRangeDigit += 2;    
             }
             
             //Check Systolic
             if(DataSystol < this.minSystolic || DataSystol > this.maxSystolic)
             {
                 WithinSetRangeDigit += 1;    
             }
             
             return WithinSetRangeDigit;
         }
         
         //If it is found that the vital signs are not within the set range
         //We can find out the offset of the vital sign. Specifically,
         //if a -ve value is given, this value is the value that is the difference
         //between the vital sign measured and the minimum vital sign set.
         //If a +ve value is given, this value is the value that is the difference
         //between the vital sign measured and the maximum vital sign set.
         public double[] CheckVitalSignsOffset() throws Exception
         {
             //{Heart Rate, Temp, Systol}
             double[] VitalSignsOffset = {0,0,0};
             double[] dblPointData = null;
             int Checks = 0;
             
             try{
                 Checks = WithinSetRange();
             }
             catch(Exception x)
             {
                 throw new Exception("Could not retrieve which data points were out of range, using WithinSetRange() in CheckVitalSignsOffset(). " + x.toString());
             }
             
             try{
                 
                 dblPointData = getPointData();
             }
             catch(Exception x)
             {
                 throw new Exception("Could not retrieve point_data as a double array while trying to determine range values in WithinSetRange(). " + x.toString());
             }
             
             
             switch(Checks)
             {
                 case 0:
                     
                     break;
                 case 1: //Systolic only                    
                     VitalSignsOffset[2] = GetDiff(dblPointData[2],this.minSystolic, this.maxSystolic);
                     break;
                 case 2: //Temperature only
                     VitalSignsOffset[1] = GetDiff(dblPointData[1], this.minTemp, this.maxTemp);
                     break;
                 case 3: //Systolic and Temperature only
                     VitalSignsOffset[2] = GetDiff(dblPointData[2],this.minSystolic, this.maxSystolic);
                     VitalSignsOffset[1] = GetDiff(dblPointData[1], this.minTemp, this.maxTemp);
                     break;
                 case 4: //Heart Rate only
                     VitalSignsOffset[0] = GetDiff(dblPointData[0], this.minHeartRate, this.maxHeartRate);
                     break;
                 case 5: //Systolic and Heart Rate only
                     VitalSignsOffset[2] = GetDiff(dblPointData[2],this.minSystolic, this.maxSystolic);
                     VitalSignsOffset[0] = GetDiff(dblPointData[0], this.minHeartRate, this.maxHeartRate);
                     break;
                 case 6: //Heart Rate and Temperature only
                     VitalSignsOffset[1] = GetDiff(dblPointData[1], this.minTemp, this.maxTemp);
                     VitalSignsOffset[0] = GetDiff(dblPointData[0], this.minHeartRate, this.maxHeartRate);
                     break;
                 case 7: //All three!
                     VitalSignsOffset[2] = GetDiff(dblPointData[2],this.minSystolic, this.maxSystolic);
                     VitalSignsOffset[1] = GetDiff(dblPointData[1], this.minTemp, this.maxTemp);
                     VitalSignsOffset[0] = GetDiff(dblPointData[0], this.minHeartRate, this.maxHeartRate);
                     break;
                     
             }
             
             return VitalSignsOffset;
         }
         
         //Helper for: CheckVitalSignsOffset()
         private double GetDiff(double dataPoint, double cutoffPointMin, double cutoffPointMax)
         {
             double diff = 0;
             
             if(dataPoint < cutoffPointMin)
                 diff = (dataPoint - cutoffPointMin);
             
             if(dataPoint > cutoffPointMax)        
                 diff = (dataPoint - cutoffPointMax);
             
             
             return diff;
                 
         }
         
                  
         //Monitoring Physician sets this if needed.
         public void SetMinMaxHeartRate(double min, double max)
         {
             this.minHeartRate = min;
             this.maxHeartRate = max;
         }
         
         //Monitoring Physician sets this if needed.
         public void SetMinMaxTemp(double min, double max)
         {
             this.minTemp = min;
             this.maxTemp = max;
         }
         
         //Monitoring Physician sets this if needed.
         public void SetMinMaxSystolic(double min, double max)
         {
             this.minSystolic = min;
             this.maxSystolic = max;
         }
         
         
         
        /**
         * Tells us the mode of trivar_normal. Two modes:  functional mode, and non-functional mode.
         *
         * Functional Mode: After we've trained trivar_normal with 24 hours worth of data,
         * we want to be able to determine if subsequent data is an outlier.
         * 
         * Non-Functional Mode: All data points being entered into trivar_normal is part of the 
         * training data. We cannot yet determine properly whether a data point is a proper outlier.
         * 
         * @return data type Boolean - True if trivar_normal is in functional mode.
         */
	public Boolean IsFunctionalMode()
	{	    		
	    return FunctionalMode;	
	}
		
        /**
         * Sets trivar_normal's mode: Functional Mode or Non-Functional Mode.
         * Give SetMode True to set into Functional Mode.
         * Give Set Mode False to set into Non-Functional Mode.
         * 
         * Functional Mode: After we've trained trivar_normal with 24 hours worth of data,
         * we want to be able to determine if subsequent data is an outlier.
         * 
         * Non-Functional Mode: All data points being entered into trivar_normal is part of the 
         * training data. We cannot yet determine properly whether a data point is a proper outlier.
         * 
         * 
         * @param mode
         */
        private void SetMode(Boolean mode)
        {
            FunctionalMode = mode;
        }
        
        /**
         * trivar_normal class has two modes: functional mode and non-functional mode.
         * 
         * When trivar_normal is in Functional Mode:
         * You are entering non-training data. This means that you want to determine
         * if the data is an outlier.
         * 
         * Order: double[] data = {a, b, c};
         * where a = Heart Rate; b = temperature; c = systolic blood pressure
         * 
         * Note: You can only determine if the data point is an outlier after you have
         * trained trivar_normal. This means 24 hours worth of data. 
         * 
         * @param data - If trivar_normal is in functional mode, this parameter is
         * the data point that we are trying to examine (whether it is an outlier).
         */
	public void setDataMatrix(double[] dbldata) throws Exception {
		
            this.point_data = new Vektor(dbldata);
                           
	}

	public Matrix getTrainingMatrix() {
		return this.training_data;
	}
	
	public double[] getPointData() throws Exception
	{
	    try{	
                return this.point_data.getDouble();
            }
            catch(JasymcaException x)
            {
                throw new Exception("Could not retrieve point_data as a double array. " + x.toString());
            }
	}	
	
		        
        /**
         * Gives the calculated mean matrix of the training data
         * Remember: The Mahalanobis distance is calculated by:
         * (datapoint - meanMatrix)^T * (CovarianceMatrix)^-1 * (datapoint-meanMatrix)
         *
         * In words:
         * The Mahalanobis distance is calculated by:
         * 1. The transpose of the matrix of the datapoint minus the mean matrix
         * 2. Inverse of the covariance matrix
         * 3. the matrix of the datapoint minus the mean matrix
         * Then, multiply the result of 1. with 2. Then multiply that result by 3.
         *  
         * @return data type Matrix
         */
	public Matrix getMeanMatrix()
	{
		return this.mean_matrix;		
	}
	
        /**
         * Gives the calculated covariance matrix of the training data
         * Remember: The Mahalanobis distance is calculated by:
         * (datapoint - meanMatrix)^T * (CovarianceMatrix)^-1 * (datapoint-meanMatrix)
         *
         * In words:
         * The Mahalanobis distance is calculated by:
         * 1. The transpose of the matrix of the datapoint minus the mean matrix
         * 2. Inverse of the covariance matrix
         * 3. the matrix of the datapoint minus the mean matrix
         * Then, multiply the result of 1. with 2. Then multiply that result by 3.
         *
         * @return data type Matrix
         */
	public Matrix getCovarianceMatrix()
	{
		return this.covar_matrix;		
	}
	
        /**
         * populates this.mean_matrix
         * @throws java.lang.Exception
         */
	private void calcMeanMatrix() throws Exception
	{
	    /**
             * In Each column of training_data:
             * 1. Sum all elements
             * 2. Divide all elements by the number of rows
             */
            
            double[][] dblTrainingData = null;
            int iNumRowsTraining = training_data.nrow();
            int iNumColsTraining = training_data.ncol();
                        
            try{
                
                dblTrainingData = training_data.getDouble(training_data.nrow(), training_data.ncol());
            }
            catch(JasymcaException x)
            {
                //We need to handle the exception when we are unable to get values from the training_data matrix 
                throw new Exception("Could not get data from training_data matrix. " + x.toString());
            }
            
            //We get down to this point if we are able to get training data
            double dblSumMean[][] = new double[1][3];
            double dblSum = 0;
            for(int c = 0; c < iNumColsTraining; c++)
            {
                for(int r = 0; r < iNumRowsTraining; r++)
                {
                    dblSum += dblTrainingData[r][c];
                }
                
                //Get the mean of the column
                dblSumMean[0][c] = dblSum/iNumRowsTraining;
                
                //Summary:
                //So right now, I know this much:
                //    Algebraic (abstract class)
                //       ^
                //       |
                //      Zahl (abstract class)
                //       ^
                //       |
                //     Exact (concrete class) 
                // I need to use Matrix.set(int i, int k, Algebraic x) in order to set a matrix element's value
                // Algebraic is an abstract class => Need to instantiate an Unexakt object. The Unexakt object is 
                // a representation to Jasymca of a primitive Java number datatype.
                // In other words, Unexakt is a number wrapper so that Jasymca can interpret numbers
                //Unexakt result = new Unexakt(dblSumMean[0][c]);
                Unexakt result = new Unexakt(dblSumMean[0][c]);
                this.mean_matrix.set(0,c,(Algebraic)result);
                
                dblSum = 0;
            }
                        
	}
	
        /**
         * populates this.covar_matrix
         * @param meanMatrix
         */
	private void calcCovarMatrix() throws Exception
	{
	    /**
             * How to calculate a 3x3 covariance matrix:
             * 
             * Note the set-up of the covariance matrix:
             * [ cov(x,x) cov(x,y) cov(x,z) ]
             * [ cov(y,x) cov(y,y) cov(y,z) ]
             * [ cov(z,x) cov(z,y) cov(z,z) ]
             * 
             * COV(x,y) = SUM( (Xi - Xm)(Yi - Ym) )/NumRowsInTraining_Data 
             * Where SUM is summing from first row to the last row.
             * 
             * An example - Steps for calculating cov(x,x):
             * 1. Get the x column in training_data
             * 2. Get the x column in the mean matrix
             * 3. For each row in the x column of the training_data, subtract from this value the
             *    x column of the mean matrix. The result will be a vector.
             * 4. Multiply this vector with the transpose of itself -> we get a scalar
             * 5. Divide the result of step 4 by the number of rows in the x column in training_data
             */		 
            
             //Create a Vektor array so that we can access the columns of the 
             //training data much easier. I say that this is easier, since there
             //is no Matrix method that allows us to get a whole column or row
             //in one go.
             Vektor[] columnVectors = CreateColumnVector(this.training_data);
             int training_data_nrows = this.training_data.nrow();
             int training_data_ncols = this.training_data.ncol();
             
             //Form: cov(dimrow,dimcol) i.e cov(x,y) to get the covariance between x and y columns
             //in training_data
             int dimrow = 0; //tells us which covariance element row to get
             int dimcol = 0; //tells us which covariance element column to get
             
             Unexakt n = new Unexakt(0);
             
             for(dimcol = 0; dimcol < training_data_ncols; dimcol++)
             {
                 for(dimrow = 0; dimrow < training_data_ncols; dimrow++)
                 {
                     try{
                         n = Cov(dimrow,dimcol, columnVectors[dimrow], columnVectors[dimcol]);
                     }
                     catch(Exception x)
                     {
                         throw new Exception("Could not obtain the covariance element. Ensure that the training data matrix has been properly created. " + x.toString());
                     }
                     
                     this.covar_matrix.set(dimrow, dimcol, n);
                 }
             }
             
             try{
                 
                //Check the contents of covar_matrix (compare w/ Matlab results) 
                double blahblah[][] = covar_matrix.getDouble(); //TEST LINE
                 
                double e00 = blahblah[0][0]; //TEST LINE
                double e01 = blahblah[0][1]; //TEST LINE
                double e02 = blahblah[0][2]; //TEST LINE
                
                double e10 = blahblah[1][0]; //TEST LINE
                double e11 = blahblah[1][1]; //TEST LINE
                double e12 = blahblah[1][2]; //TEST LINE
                
                double e20 = blahblah[2][0]; //TEST LINE
                double e21 = blahblah[2][1]; //TEST LINE
                double e22 = blahblah[2][2]; //TEST LINE
                
                //Check the contents of mean vector (compare w/ Matlab results)
                double meanblah[][] = this.mean_matrix.getDouble();
                
                double m00 = meanblah[0][0];
                double m01 = meanblah[0][1];
                double m02 = meanblah[0][2];
                int meanrows = this.mean_matrix.nrow();
                int meancol = this.mean_matrix.ncol();  
                
                
                 //Remember!: Linear independence of the covariance matrix is 
                 //necessary to get the 
                 //inverse of the covariance matrix here.
                 covarMatInverse = covar_matrix.invert();
                
                //Check the result of the inversion of covar_matrix
                //(compare w/ Matlab results)
                blahblah = covarMatInverse.getDouble(); //TEST LINE
                 
                e00 = blahblah[0][0]; //TEST LINE
                e01 = blahblah[0][1]; //TEST LINE
                e02 = blahblah[0][2]; //TEST LINE
                
                e10 = blahblah[1][0]; //TEST LINE
                e11 = blahblah[1][1]; //TEST LINE
                e12 = blahblah[1][2]; //TEST LINE
                
                e20 = blahblah[2][0]; //TEST LINE
                e21 = blahblah[2][1]; //TEST LINE
                e22 = blahblah[2][2]; //TEST LINE
                
                 
             }
             catch(JasymcaException x)
             {
                 throw new Exception("Could not get the inverse of the covariance matrix" + x.toString());
             }
	}
	
        /**
         * cov(dimension,dimension), where dimension is x, y, or z
         * Calculates the covariance element to place into the covariance matrix
         * @param dimrow => x = 0; y = 1; z = 2; (this tells us which column in mean matrix to use)
         * @param dimcol => x = 0; y = 1; z = 2; (this tells us which column in mean matrix to use)
         * @return
         */
        private Unexakt Cov(int dimrow, int dimcol, Vektor RowVector, Vektor ColVector) throws Exception
        {
             /**
             * EXAMPLE:
             * COV(x,y) = SUM( (Xi - Xm)(Yi - Ym) )/NumRowsInTraining_Data 
             * Where SUM is summing from first row to the last row.
             */
            
             int training_data_nrows = this.training_data.nrow();
             Vektor interColVector = null;
             Vektor interRowVector = null;
             double Sum = 0;
             Unexakt ColResult = null; //This is the result of the subtraction of Xi - Xm
             Unexakt RowResult = null; //This is the result of the subtraction of Yi - Ym
             Unexakt result = new Unexakt(0);
             
             //for each row in the column of the training data, subtract from this value
             //the matching column of the mean matrix. Store the result in the
             //generic covar matrix (Note: it is a matrix that is of dimension training_data_nrows,1
             for(int j = 0; j < training_data_nrows; j++)
             {       
                 try{
                     
                     //Just remember: Vektor extends Algebraic
                     //you can't cast an Algebraic into a Vektor. You can only cast
                     //a Vektor into an Algebraic.
                     //interColVector = (new Vektor( ColVector.get(j).sub(this.mean_matrix.get(0, dimcol)) )).getDouble();
                                          
                     //subCol[] will always be a one element vektor
                     
                     double subCol[] = (new Vektor( ColVector.get(j).sub(this.mean_matrix.get(0, dimcol)) )).getDouble();
                     
                     ColResult = new Unexakt(subCol[0]);
                     
                     double meanMatrix[] = new Vektor(this.mean_matrix.get(0,dimcol)).getDouble();//TEST LINE
                     double meanMatrixE1 = meanMatrix[0]; //TEST LINE
                    
                     double blah = ColResult.real; //TEST LINE
                     double blah1 = ColResult.imag; //TEST LINE
                 }
                 catch(JasymcaException x)
                 {
                     throw new Exception("Could not perform row subtraction on column dimension for obtaining covariance element. " + x.toString());
                 }
                 
                 try{
                     double subRow[] = (new Vektor(RowVector.get(j).sub(this.mean_matrix.get(0, dimrow)))).getDouble();
                     RowResult = new Unexakt(subRow[0]);
                     
                     double meanMatrix[] = new Vektor(this.mean_matrix.get(0,dimrow)).getDouble();//TEST LINE
                     double meanMatrixE1 = meanMatrix[0]; //TEST LINE
                                         
                     
                     double blah = RowResult.real; //TEST LINE
                     double blah2 = RowResult.imag; //TEST LINE
                     blah = 0; //TESTLINE
                 }
                 catch(JasymcaException x)
                 {
                     throw new Exception("Could not perform row subtraction on row dimension for obtaining covariance element. " + x.toString());
                 }
                 
                 //do: (Xi - Xm)(Yi - Ym) and divide
                 //by the number of rows in training data.
                 //Just remember: Vektor extends Algebraic
                 //you can't cast an Algebraic into a Vektor. You can only cast
                 //a Vektor into an Algebraic.
                 double multResult[] = (new Vektor(ColResult.mult(RowResult))).getDouble(); //TEST LINE
                 Unexakt multResultU = new Unexakt(multResult[0]); //TEST LINE
                                
                 Sum += multResult[0]; 
                 
                 
             }//for
             
             if(training_data_nrows != 0)
             {
                 result = new Unexakt(Sum/(training_data_nrows-1));
                 
             }
             else
             {
                 throw new Exception("Attempted to divide zero number of training data rows in order to get a covariance matrix element. Check if data has been set properly.");
             }
             double test = result.real; //TEST LINE
             double test2 = result.imag; //TEST LINE
             return result;
             
        }
        
        /**
         * A helper method that allows us to access the columns of a matrix 
         * very easily.
         * @param the matrix that we would like to break up.
         * @return an array of Vektors that represent a set of columns of this method's parameter
         */
        private Vektor[] CreateColumnVector(Matrix matrix) throws Exception
        {
            int nCol = matrix.ncol();
            int nRow = matrix.nrow();
            Vektor[] result = new Vektor[nCol];
            
            //initialize the vektors within result array
            for(int i = 0; i < nCol; i++)
            {
                result[i] = new Vektor(nRow);
            }
                        
            for(int vectorArray = 0; vectorArray < nCol; vectorArray++)
            {
                for(int row = 0; row < nRow; row++)
                {
                    try{
                        result[vectorArray].set(row, matrix.get(row, vectorArray) );
                    }
                    catch(JasymcaException x)
                    {
                        throw new Exception("Could not create a set of column vectors from the input matrix. " + x.toString());
                    }
                }
            }
            
            return result;
        }
                	
        /** mahalanobisDist() 
          * Calculates the Mahalanobis distance of an incoming data point.
          * An incoming data point is a vector that has three values:
          * 1. Heart rate
          * 2. Temperature
          * 3. Blood Pressure 
          */
	private double mahalanobisDist() throws Exception
	{
	     Vektor pointDataMinusMeanVec = null;
             
             
             //Calculate the Mahalanobis distance
             try{
                 
                 int lenPointDataVektor = this.point_data.length(); // TEST LINE
                 Vektor mean_vektor = new Vektor(this.mean_matrix.row(1));
                 
                 pointDataMinusMeanVec = new Vektor(this.point_data.sub(mean_vektor));
                 
                 double pointDatablah[] = pointDataMinusMeanVec.getDouble();//TEST LINE
                 double p0 = pointDatablah[0]; //TEST LINE
                 double p1 = pointDatablah[1]; //TEST LINE
                 double p2 = pointDatablah[2]; //TEST LINE
                 int pointDatarows = pointDataMinusMeanVec.length(); //TEST LINE
                 int thing = 0; //TEST LINE
                 //We know that the point data minus mean vector result is
                 //the correct one
             }
             catch(JasymcaException x)
             {
                throw new Exception("Could not subtract the point data from the mean vector" + x.toString());
             }
                                   
             
             //Note: 1x3 matrix multiplied by 3x3 matrix => 1x3 matrix
             Matrix intermediate = new Matrix((new Matrix(pointDataMinusMeanVec)).mult(covarMatInverse));
             
             double iblah[][] = intermediate.getDouble();
             int inrows = intermediate.nrow();  //TEST LINE
             int incols = intermediate.ncol();  //TEST LINE
             double i00 = iblah[0][0]; //TEST LINE
             double i01 = iblah[0][1]; //TEST LINE
             double i02 = iblah[0][2]; //TEST LINE
             
             //Transpose the pointData matrix before multiplying with the intermediate result!
             //since: intermediate result is 1x3 and so we want point data to be 3x1
             Matrix pointDataMinusMeanVecMatrix = (new Matrix(pointDataMinusMeanVec)).transpose(); 
             
             Vektor distanceVek = new Vektor((intermediate.mult( pointDataMinusMeanVecMatrix )));
                          
             double dist_scalar_sq = distanceVek.norm(); //We want to be able to obtain the magnitude of the vektor which is just the scalar value
             
             return dist_scalar_sq;
	}
	
}

